/*****************************************************************************
 * AesExamples.h
 *****************************************************************************/

#ifndef __AESEXAMPLES_H__
#define __AESEXAMPLES_H__

/* macro definitions */
#define AES_IV_SZ (128/8)
#define BYPASS_SZ_WDS 16

/* structure definition for Command and Result Descriptor */
typedef struct{
  unsigned long CTRL_STAT;
  unsigned long SRC_ADDR;
  unsigned long DEST_ADDR;
  unsigned long SA_ADDR;
  unsigned long SA_STATE_ADDR;
#if defined(__ADSPSC58x__) && __ADSPSC58x__==1
  unsigned long ARC4_STATE_ADDR;
#else
  unsigned long RESERVED;
#endif
  unsigned long USERID;
  unsigned long LENGTH;
} cd_t, rd_t;

/* structure definition for Security Association (SA) Record */
typedef struct{
  unsigned long CMD_0;
  unsigned long CMD_1;
  unsigned long KEY_0;
  unsigned long KEY_1;
  unsigned long KEY_2;
  unsigned long KEY_3;
  unsigned long KEY_4;
  unsigned long KEY_5;
  unsigned long KEY_6;
  unsigned long KEY_7;
  unsigned long IDIGEST_0;
  unsigned long IDIGEST_1;
  unsigned long IDIGEST_2;
  unsigned long IDIGEST_3;
  unsigned long IDIGEST_4;
  unsigned long IDIGEST_5;
  unsigned long IDIGEST_6;
  unsigned long IDIGEST_7;
  unsigned long ODIGEST_0;
  unsigned long ODIGEST_1;
  unsigned long ODIGEST_2;
  unsigned long ODIGEST_3;
  unsigned long ODIGEST_4;
  unsigned long ODIGEST_5;
  unsigned long ODIGEST_6;
  unsigned long ODIGEST_7;
  unsigned long SPI;
  unsigned long SEQNUM_0;
  unsigned long SEQNUM_1;
  unsigned long SEQNUMMSK_0;
  unsigned long SEQNUMMSK_1;
  unsigned long NONCE;
} sa_t;


/* structure definition for Security Association (SA) State Record  */
typedef struct{
  unsigned long STATE_IV_0;
  unsigned long STATE_IV_1;
  unsigned long STATE_IV_2;
  unsigned long STATE_IV_3;
  unsigned long STATE_BYTE_CNT_0;
  unsigned long STATE_BYTE_CNT_1;
  unsigned long STATE_IDIGEST_0;
  unsigned long STATE_IDIGEST_1;
  unsigned long STATE_IDIGEST_2;
  unsigned long STATE_IDIGEST_3;
  unsigned long STATE_IDIGEST_4;
  unsigned long STATE_IDIGEST_5;
  unsigned long STATE_IDIGEST_6;
  unsigned long STATE_IDIGEST_7;
} state_t;

#define PKTE_BUFFER_SIZE 256

/* enum for IV source */
enum SA_CMD0_IV_SRC{
	no_load_iv,
	iv_from_inbuf,
	iv_from_state,
	iv_from_prng
};

/* enum for hash function */
enum SA_CMD0_HASH{
#if defined(__ADSPSC58x__) && __ADSPSC58x__==1
	md5,
#else
	sa_cmd0_hash_rsv0,
#endif
	sha1,
	sha224,
	sha256,
	null_hash = 15
};

/* enum for cipher algorithm */
enum SA_CMD0_CIPHER{
	des,
	tdes,
#if defined(__ADSPSC58x__) && __ADSPSC58x__==1
	arc4,
#else
	sa_cmd0_cipher_rsv,
#endif
	aes,
	null_cipher = 15
};

/* enum for pad type */
enum SA_CMD0_PADTYPE{
	ipsec,
	pkcs7,
	constant,
	zero,
	sa_cmd0_padtype_rsv,
	tls,
	constant_ssl
};

/* enum for operation group */
enum SA_CMD0_OPGROUP{
	basic,
	protocol,
	ext_prot
};

/* enum for operation direction */
enum SA_CMD0_DIR{
	outbound,
	inbound
};

/* enum for opcode when operation is outbound */
enum SA_CMD0_OPCODE_OUTBOUND{
	encrypt,
	encrypt_hash,
	sa_cmd0_opcode_outbound_rsv,
	out_hash,
	prng = 7
};


/* enum for opcode when operation is inbound */
enum SA_CMD0_OPCODE_INBOUND{
	decrypt,
	hash_decrypt,
	sa_cmd0_opcode_inbound_rsv,
	in_hash,
};

/* enum for keylength */
enum SA_CMD1_AES_KEY_LENGTH{
	bits_128 = 2,
	bits_192,
	bits_256
};

/* enum for cipher mode */
enum SA_CMD1_CIPHER_MODE{
	ecb,
	cbc,
	ctr,
	icm
};


#ifndef __ADSPSHARC__
#if defined(section)
#undef section
#endif
#define section(...)
#endif

#if defined(__ADSPSHARC__)
#define ssync() asm("sync;")
#elif defined(__ADSPGCC__)
#define ssync() asm("DSB")
#endif



#endif /* __AESEXAMPLES_H__ */
